# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ciff.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\nciff.proto\x12\x0eio.osirrc.ciff\"\xcc\x01\n\x06Header\x12\x0f\n\x07version\x18\x01 \x01(\x05\x12\x1a\n\x12num_postings_lists\x18\x02 \x01(\x05\x12\x10\n\x08num_docs\x18\x03 \x01(\x05\x12\x1c\n\x14total_postings_lists\x18\x04 \x01(\x05\x12\x12\n\ntotal_docs\x18\x05 \x01(\x05\x12!\n\x19total_terms_in_collection\x18\x06 \x01(\x03\x12\x19\n\x11\x61verage_doclength\x18\x07 \x01(\x01\x12\x13\n\x0b\x64\x65scription\x18\x08 \x01(\t\"$\n\x07Posting\x12\r\n\x05\x64ocid\x18\x01 \x01(\x05\x12\n\n\x02tf\x18\x02 \x01(\x05\"_\n\x0cPostingsList\x12\x0c\n\x04term\x18\x01 \x01(\t\x12\n\n\x02\x64\x66\x18\x02 \x01(\x03\x12\n\n\x02\x63\x66\x18\x03 \x01(\x03\x12)\n\x08postings\x18\x04 \x03(\x0b\x32\x17.io.osirrc.ciff.Posting\"G\n\tDocRecord\x12\r\n\x05\x64ocid\x18\x01 \x01(\x05\x12\x18\n\x10\x63ollection_docid\x18\x02 \x01(\t\x12\x11\n\tdoclength\x18\x03 \x01(\x05\x62\x06proto3')



_HEADER = DESCRIPTOR.message_types_by_name['Header']
_POSTING = DESCRIPTOR.message_types_by_name['Posting']
_POSTINGSLIST = DESCRIPTOR.message_types_by_name['PostingsList']
_DOCRECORD = DESCRIPTOR.message_types_by_name['DocRecord']
Header = _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), {
  'DESCRIPTOR' : _HEADER,
  '__module__' : 'ciff_pb2'
  # @@protoc_insertion_point(class_scope:io.osirrc.ciff.Header)
  })
_sym_db.RegisterMessage(Header)

Posting = _reflection.GeneratedProtocolMessageType('Posting', (_message.Message,), {
  'DESCRIPTOR' : _POSTING,
  '__module__' : 'ciff_pb2'
  # @@protoc_insertion_point(class_scope:io.osirrc.ciff.Posting)
  })
_sym_db.RegisterMessage(Posting)

PostingsList = _reflection.GeneratedProtocolMessageType('PostingsList', (_message.Message,), {
  'DESCRIPTOR' : _POSTINGSLIST,
  '__module__' : 'ciff_pb2'
  # @@protoc_insertion_point(class_scope:io.osirrc.ciff.PostingsList)
  })
_sym_db.RegisterMessage(PostingsList)

DocRecord = _reflection.GeneratedProtocolMessageType('DocRecord', (_message.Message,), {
  'DESCRIPTOR' : _DOCRECORD,
  '__module__' : 'ciff_pb2'
  # @@protoc_insertion_point(class_scope:io.osirrc.ciff.DocRecord)
  })
_sym_db.RegisterMessage(DocRecord)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _HEADER._serialized_start=31
  _HEADER._serialized_end=235
  _POSTING._serialized_start=237
  _POSTING._serialized_end=273
  _POSTINGSLIST._serialized_start=275
  _POSTINGSLIST._serialized_end=370
  _DOCRECORD._serialized_start=372
  _DOCRECORD._serialized_end=443
# @@protoc_insertion_point(module_scope)
